import uuid
from typing import TYPE_CHECKING, List, Optional, Dict

from .models import (
    QualGenerationRequest,
    QualGenerationResponse,
    MCGenerationRequest,
    MCGenerationResponse,
    SurveySessionCloseResponse,
    AddContextRequest,
    AddContextResponse,
    SurveySessionDetailResponse,
)

if TYPE_CHECKING:
    from .client import Simile


class Agent:
    """Represents an agent and provides methods for interacting with it directly."""

    def __init__(self, agent_id: uuid.UUID, client: "Simile"):
        self._agent_id = agent_id
        self._client = client

    @property
    def id(self) -> uuid.UUID:
        return self._agent_id

    async def generate_qual_response(
        self, question: str, image_url: Optional[str] = None
    ) -> QualGenerationResponse:
        """Generates a qualitative response from this agent based on a question."""
        return await self._client.generate_qual_response(
            agent_id=self._agent_id, question=question, image_url=image_url
        )

    async def generate_mc_response(
        self, question: str, options: List[str], image_url: Optional[str] = None
    ) -> MCGenerationResponse:
        """Generates a multiple-choice response from this agent."""
        return await self._client.generate_mc_response(
            agent_id=self._agent_id,
            question=question,
            options=options,
            image_url=image_url,
        )


class SurveySession:
    """Represents an active survey session with an agent, allowing for contextual multi-turn generation."""

    def __init__(
        self, id: uuid.UUID, agent_id: uuid.UUID, status: str, client: "Simile"
    ):
        self._id = id
        self._agent_id = agent_id
        self._status = status
        self._client = client

    @property
    def id(self) -> uuid.UUID:
        return self._id

    @property
    def agent_id(self) -> uuid.UUID:
        return self._agent_id

    @property
    def status(self) -> str:
        return self._status

    async def get_details(self) -> SurveySessionDetailResponse:
        """Retrieves detailed information about this survey session including typed conversation history."""
        return await self._client.get_survey_session(self._id)

    async def generate_qual_response(
        self,
        question: str,
        image_url: Optional[str] = None,
        images: Optional[Dict[str, str]] = None,
    ) -> QualGenerationResponse:
        """Generates a qualitative response within this survey session."""
        endpoint = f"sessions/{str(self._id)}/qual"
        payload = QualGenerationRequest(
            question=question, image_url=image_url, images=images
        )
        return await self._client._request(
            "POST",
            endpoint,
            json=payload.model_dump(),
            response_model=QualGenerationResponse,
        )

    async def generate_mc_response(
        self, question: str, options: List[str], image_url: Optional[str] = None
    ) -> MCGenerationResponse:
        """Generates a multiple-choice response within this survey session."""
        endpoint = f"sessions/{str(self._id)}/mc"
        payload = MCGenerationRequest(
            question=question, options=options, image_url=image_url
        )
        return await self._client._request(
            "POST",
            endpoint,
            json=payload.model_dump(),
            response_model=MCGenerationResponse,
        )

    async def add_context(self, ctx: str) -> AddContextResponse:
        """Adds text to the SurveySession without requesting a response."""
        endpoint = f"sessions/{str(self._id)}/context"
        payload = AddContextRequest(context=ctx)
        return await self._client._request(
            "POST",
            endpoint,
            json=payload.model_dump(),
            response_model=AddContextResponse,
        )

    async def close(self) -> SurveySessionCloseResponse:
        """Closes this survey session on the server."""
        endpoint = f"sessions/{str(self._id)}/close"
        return await self._client._request(
            "POST", endpoint, response_model=SurveySessionCloseResponse
        )
